@@ -9,6 +9,25 @@ use Icinga\Exception\IcingaException;
 
 class WebCommand extends Command
 {
+    /**
+     * Serve Icinga Web 2 with PHP's built-in web server
+     *
+     * USAGE
+     *
+     *   icingacli web serve [options] [<document-root>]
+     *
+     * OPTIONS
+     *
+     *   --daemonize            Run in background
+     *   --port=<port>          The port to listen on
+     *   --listen=<host:port>   The address to listen on
+     *   <document-root>        The document root directory of Icinga Web 2 (e.g. ./public)
+     *
+     * EXAMPLES
+     *
+     *   icingacli web serve --port=8080
+     *   icingacli web serve --listen=127.0.0.1:8080 ./public
+     */
     public function serveAction()
     {
         $minVersion = '5.4.0';
@@ -21,13 +40,17 @@ class WebCommand extends Command
         }
 
         $fork = $this->params->get('daemonize');
+        $listen = $this->params->get('listen');
+        $port = $this->params->get('port');
         $documentRoot = $this->params->shift();
-        $socket  = $this->params->shift();
+        if ($listen === null) {
+            $socket = $port === null ? $this->params->shift() : '0.0.0.0:' . $port;
+        } else {
+            $socket = $listen;
+        }
 
-        // TODO: Sanity check!!
         if ($socket === null) {
-            $socket = '0.0.0.0:80';
-            // throw new IcingaException('Socket is required');
+            $socket = $this->Config()->get('standalone', 'listen', '0.0.0.0:80');
         }
         if ($documentRoot === null) {
             $documentRoot = Icinga::app()->getBaseDir('public');
@@ -40,7 +63,7 @@ class WebCommand extends Command
         if ($fork) {
             $this->forkAndExit();
         }
-        echo "Serving Icingaweb from $documentRoot\n";
+        echo "Serving Icinga Web 2 from directory $documentRoot and listening on $socket\n";
         $cmd = sprintf(
             '%s -S %s -t %s %s',
             readlink('/proc/self/exe'),
@@ -69,7 +92,7 @@ class WebCommand extends Command
         $pid = pcntl_fork();
         if ($pid == -1) {
              throw new IcingaException('Could not fork');
-        } else if ($pid) {
+        } elseif ($pid) {
             echo $this->screen->colorize('[OK]')
                . " Icinga Web server forked successfully\n";
             fclose(STDIN);
