@@ -3,12 +3,11 @@
 
 namespace Icinga\Controllers;
 
+use Icinga\Exception\IcingaException;
 use Zend_Controller_Plugin_ErrorHandler;
 use Icinga\Application\Icinga;
 use Icinga\Application\Logger;
-use Icinga\Exception\Http\HttpBadRequestException;
-use Icinga\Exception\Http\HttpMethodNotAllowedException;
-use Icinga\Exception\Http\HttpNotFoundException;
+use Icinga\Exception\Http\HttpExceptionInterface;
 use Icinga\Exception\MissingParameterException;
 use Icinga\Security\SecurityException;
 use Icinga\Web\Controller\ActionController;
@@ -24,6 +23,14 @@ class ErrorController extends ActionController
      */
     protected $requiresAuthentication = false;
 
+    /**
+     * {@inheritdoc}
+     */
+    public function init()
+    {
+        $this->rerenderLayout = $this->params->has('renderLayout');
+    }
+
     /**
      * Display exception
      */
@@ -32,8 +39,6 @@ class ErrorController extends ActionController
         $error      = $this->_getParam('error_handler');
         $exception  = $error->exception;
         /** @var \Exception $exception */
-        Logger::error($exception);
-        Logger::error('Stacktrace: %s', $exception->getTraceAsString());
 
         if (! ($isAuthenticated = $this->Auth()->isAuthenticated())) {
             $this->innerLayout = 'guest-error';
@@ -61,12 +66,11 @@ class ErrorController extends ActionController
                 break;
             default:
                 switch (true) {
-                    case $exception instanceof HttpMethodNotAllowedException:
-                        $this->getResponse()->setHttpResponseCode(405);
-                        $this->getResponse()->setHeader('Allow', $exception->getAllowedMethods());
-                        break;
-                    case $exception instanceof HttpNotFoundException:
-                        $this->getResponse()->setHttpResponseCode(404);
+                    case $exception instanceof HttpExceptionInterface:
+                        $this->getResponse()->setHttpResponseCode($exception->getStatusCode());
+                        foreach ($exception->getHeaders() as $name => $value) {
+                            $this->getResponse()->setHeader($name, $value, true);
+                        }
                         break;
                     case $exception instanceof MissingParameterException:
                         $this->getResponse()->setHttpResponseCode(400);
@@ -75,19 +79,30 @@ class ErrorController extends ActionController
                             'Missing parameter ' . $exception->getParameter()
                         );
                         break;
-                    case $exception instanceof HttpBadRequestException:
-                        $this->getResponse()->setHttpResponseCode(400);
-                        break;
                     case $exception instanceof SecurityException:
                         $this->getResponse()->setHttpResponseCode(403);
                         break;
                     default:
                         $this->getResponse()->setHttpResponseCode(500);
+                        Logger::error("%s\n%s", $exception, IcingaException::getConfidentialTraceAsString($exception));
                         break;
                 }
-                $this->view->message = $exception->getMessage();
+
+                $this->view->messages = array();
+
                 if ($this->getInvokeArg('displayExceptions')) {
-                    $this->view->stackTrace = $exception->getTraceAsString();
+                    $this->view->stackTraces = array();
+
+                    do {
+                        $this->view->messages[] = $exception->getMessage();
+                        $this->view->stackTraces[] = IcingaException::getConfidentialTraceAsString($exception);
+                        $exception = $exception->getPrevious();
+                    } while ($exception !== null);
+                } else {
+                    do {
+                        $this->view->messages[] = $exception->getMessage();
+                        $exception = $exception->getPrevious();
+                    } while ($exception !== null);
                 }
 
                 break;
