@@ -4,6 +4,8 @@
 namespace Icinga\Forms\Config\General;
 
 use Icinga\Application\Logger;
+use Icinga\Application\Logger\Writer\SyslogWriter;
+use Icinga\Application\Platform;
 use Icinga\Web\Form;
 
 /**
@@ -37,6 +39,7 @@ class LoggingConfigForm extends Form
                 'label'         => $this->translate('Logging Type'),
                 'description'   => $this->translate('The type of logging to utilize.'),
                 'multiOptions'  => array(
+                    'php'       => $this->translate('Webserver Log', 'app.config.logging.type'),
                     'syslog'    => 'Syslog',
                     'file'      => $this->translate('File', 'app.config.logging.type'),
                     'none'      => $this->translate('None', 'app.config.logging.type')
@@ -62,7 +65,7 @@ class LoggingConfigForm extends Form
             );
         }
 
-        if (false === isset($formData['logging_log']) || $formData['logging_log'] === 'syslog') {
+        if (! isset($formData['logging_log']) || in_array($formData['logging_log'], array('syslog', 'php'))) {
             $this->addElement(
                 'text',
                 'logging_application',
@@ -70,7 +73,7 @@ class LoggingConfigForm extends Form
                     'required'      => true,
                     'label'         => $this->translate('Application Prefix'),
                     'description'   => $this->translate(
-                        'The name of the application by which to prefix syslog messages.'
+                        'The name of the application by which to prefix log messages.'
                     ),
                     'requirement'   => $this->translate('The application prefix must not contain whitespace.'),
                     'value'         => 'icingaweb2',
@@ -90,22 +93,33 @@ class LoggingConfigForm extends Form
                     )
                 )
             );
-            /*
-             * Note(el): Since we provide only one possible value for the syslog facility, I opt against exposing
-             * this configuration.
-             */
-//            $this->addElement(
-//                'select',
-//                'logging_facility',
-//                array(
-//                    'required'      => true,
-//                    'label'         => $this->translate('Facility'),
-//                    'description'   => $this->translate('The syslog facility to utilize.'),
-//                    'multiOptions'  => array(
-//                        'user' => 'LOG_USER'
-//                    )
-//                )
-//            );
+
+            if (isset($formData['logging_log']) && $formData['logging_log'] === 'syslog') {
+                if (Platform::isWindows()) {
+                    /* @see https://secure.php.net/manual/en/function.openlog.php */
+                    $this->addElement(
+                        'hidden',
+                        'logging_facility',
+                        array(
+                            'value' => 'user',
+                            'disabled' => true
+                        )
+                    );
+                } else {
+                    $facilities = array_keys(SyslogWriter::$facilities);
+                    $this->addElement(
+                        'select',
+                        'logging_facility',
+                        array(
+                            'required' => true,
+                            'label' => $this->translate('Facility'),
+                            'description' => $this->translate('The syslog facility to utilize.'),
+                            'value' => 'user',
+                            'multiOptions' => array_combine($facilities, $facilities)
+                        )
+                    );
+                }
+            }
         } elseif (isset($formData['logging_log']) && $formData['logging_log'] === 'file') {
             $this->addElement(
                 'text',
