@@ -156,6 +156,16 @@ class PreferenceForm extends Form
      */
     public function createElements(array $formData)
     {
+        if (setlocale(LC_ALL, 0) === 'C') {
+            $this->warning(
+                $this->translate(
+                    'Your language setting is not applied because your platform is missing the corresponding locale.'
+                    . ' Make sure to install the correct language pack and restart your web server afterwards.'
+                ),
+                false
+            );
+        }
+
         if (! (bool) Config::app()->get('themes', 'disabled', false)) {
             $themes = Icinga::app()->getThemes();
             if (count($themes) > 1) {
@@ -170,7 +180,9 @@ class PreferenceForm extends Form
                         'label'         => $this->translate('Theme', 'Form element label'),
                         'multiOptions'  => $themes,
                         'value'         => $this->preferences->getValue(
-                            'icingaweb', 'theme', $defaultTheme
+                            'icingaweb',
+                            'theme',
+                            $defaultTheme
                         )
                     )
                 );
@@ -178,13 +190,21 @@ class PreferenceForm extends Form
         }
 
         $languages = array();
-        $languages['autodetect'] = sprintf($this->translate('Browser (%s)', 'preferences.form'), $this->getLocale());
+
+        $locale = $this->getLocale();
+        if ($locale !== null) {
+            $languages['autodetect'] = sprintf($this->translate('Browser (%s)', 'preferences.form'), $locale);
+        }
+
         foreach (Translator::getAvailableLocaleCodes() as $language) {
             $languages[$language] = $language;
         }
 
         $tzList = array();
-        $tzList['autodetect'] = sprintf($this->translate('Browser (%s)', 'preferences.form'), $this->getDefaultTimezone());
+        $tzList['autodetect'] = sprintf(
+            $this->translate('Browser (%s)', 'preferences.form'),
+            $this->getDefaultTimezone()
+        );
         foreach (DateTimeZone::listIdentifiers() as $tz) {
             $tzList[$tz] = $tz;
         }
@@ -241,11 +261,23 @@ class PreferenceForm extends Form
             array(
                 'required'      => false,
                 'label'         => $this->translate('Enable auto refresh'),
-                'description'   => $this->translate('This option allows you to enable or to disable the global page content auto refresh'),
+                'description'   => $this->translate(
+                    'This option allows you to enable or to disable the global page content auto refresh'
+                ),
                 'value'         => 1
             )
         );
 
+        $this->addElement(
+            'number',
+            'default_page_size',
+            array(
+                'label'         => $this->translate('Default page size'),
+                'description'   => $this->translate('Default number of items per page for list views'),
+                'step'          => 1
+            )
+        );
+
         if ($this->store) {
             $this->addElement(
                 'submit',
@@ -310,12 +342,13 @@ class PreferenceForm extends Form
     /**
      * Return the preferred locale based on the given HTTP header and the available translations
      *
-     * @return string
+     * @return string|null
      */
     protected function getLocale()
     {
-        $locale = Translator::getPreferredLocaleCode($_SERVER['HTTP_ACCEPT_LANGUAGE']);
-        return $locale;
+        return isset($_SERVER['HTTP_ACCEPT_LANGUAGE'])
+            ? Translator::getPreferredLocaleCode($_SERVER['HTTP_ACCEPT_LANGUAGE'])
+            : null;
     }
 
     /**
