@@ -302,9 +302,18 @@ class Web extends EmbeddedWeb
                     ),
                     'children'  => array(
                         'about' => array(
-                            'label'     => t('About'),
-                            'url'       => 'about',
-                            'priority'  => 701
+                            'icon'        => 'info',
+                            'description' => t('Open about page'),
+                            'label'       => t('About'),
+                            'url'         => 'about',
+                            'priority'    => 700
+                        ),
+                        'announcements' => array(
+                            'icon'        => 'megaphone',
+                            'description' => t('List announcements'),
+                            'label'       => t('Announcements'),
+                            'url'         => 'announcements',
+                            'priority'    => 710
                         )
                     )
                 ),
@@ -315,28 +324,36 @@ class Web extends EmbeddedWeb
                     'priority'      => 800,
                     'children'      => array(
                         'application' => array(
-                            'label'         => t('Application'),
-                            'url'           => 'config/general',
-                            'permission'    => 'config/application/*',
-                            'priority'      => 810
+                            'icon'        => 'wrench',
+                            'description' => t('Open application configuration'),
+                            'label'       => t('Application'),
+                            'url'         => 'config/general',
+                            'permission'  => 'config/application/*',
+                            'priority'    => 810
                         ),
                         'authentication' => array(
-                            'label'         => t('Authentication'),
-                            'permission'    => 'config/authentication/*',
-                            'priority'      => 830,
-                            'url'           => 'role/list'
+                            'icon'        => 'users',
+                            'description' => t('Open authentication configuration'),
+                            'label'       => t('Authentication'),
+                            'permission'  => 'config/authentication/*',
+                            'priority'    => 830,
+                            'url'         => 'role/list'
                         ),
                         'navigation' => array(
-                            'label'         => t('Shared Navigation'),
-                            'url'           => 'navigation/shared',
-                            'permission'    => 'config/application/navigation',
-                            'priority'      => 840,
+                            'icon'        => 'sitemap',
+                            'description' => t('Open shared navigation configuration'),
+                            'label'       => t('Shared Navigation'),
+                            'url'         => 'navigation/shared',
+                            'permission'  => 'config/application/navigation',
+                            'priority'    => 840,
                         ),
                         'modules' => array(
-                            'label'         => t('Modules'),
-                            'url'           => 'config/modules',
-                            'permission'    => 'config/modules',
-                            'priority'      => 890
+                            'icon'        => 'cubes',
+                            'description' => t('Open module configuration'),
+                            'label'       => t('Modules'),
+                            'url'         => 'config/modules',
+                            'permission'  => 'config/modules',
+                            'priority'    => 890
                         )
                     )
                 ),
@@ -346,16 +363,20 @@ class Web extends EmbeddedWeb
                     'icon'      => 'user',
                     'priority'  => 900,
                     'children'  => array(
-                        'preferences' => array(
-                            'label'     => t('Preferences'),
-                            'priority'  => 100,
-                            'url'       => 'preference'
+                        'account' => array(
+                            'icon'        => 'sliders',
+                            'description' => t('Open your account preferences'),
+                            'label'       => t('My Account'),
+                            'priority'    => 100,
+                            'url'         => 'account'
                         ),
                         'logout' => array(
-                            'label'         => t('Logout'),
-                            'priority'      => 200,
-                            'attributes'    => array('target' => '_self'),
-                            'url'           => 'authentication/logout'
+                            'icon'        => 'off',
+                            'description' => t('Log out'),
+                            'label'       => t('Logout'),
+                            'priority'    => 200,
+                            'attributes'  => array('target' => '_self'),
+                            'url'         => 'authentication/logout'
                         )
                     )
                 )
@@ -363,10 +384,12 @@ class Web extends EmbeddedWeb
 
             if (Logger::writesToFile()) {
                 $menu['system']['children']['application_log'] = array(
-                    'label'      => t('Application Log'),
-                    'url'        => 'list/applicationlog',
-                    'permission' => 'application/log',
-                    'priority'   => 710
+                    'icon'        => 'doc-text',
+                    'description' => t(''),
+                    'label'       => t('Application Log'),
+                    'url'         => 'list/applicationlog',
+                    'permission'  => 'application/log',
+                    'priority'    => 900
                 );
             }
         } else {
@@ -411,10 +434,28 @@ class Web extends EmbeddedWeb
     private function setupUser()
     {
         $auth = Auth::getInstance();
+        if (! $this->request->isXmlHttpRequest() && $this->request->isApiRequest() && ! $auth->isAuthenticated()) {
+            $auth->authHttp();
+        }
         if ($auth->isAuthenticated()) {
             $user = $auth->getUser();
             $this->getRequest()->setUser($user);
             $this->user = $user;
+
+            if ($user->can('application/stacktraces')) {
+                $displayExceptions = $this->user->getPreferences()->getValue(
+                    'icingaweb',
+                    'show_stacktraces'
+                );
+
+                if ($displayExceptions !== null) {
+                    $this->frontController->setParams(
+                        array(
+                            'displayExceptions' => $displayExceptions
+                        )
+                    );
+                }
+            }
         }
         return $this;
     }
@@ -454,13 +495,6 @@ class Web extends EmbeddedWeb
         $this->frontController->setControllerDirectory($this->getApplicationDir('/controllers'));
 
         $displayExceptions = $this->config->get('global', 'show_stacktraces', true);
-        if ($this->user !== null && $this->user->can('application/stacktraces')) {
-            $displayExceptions = $this->user->getPreferences()->getValue(
-                'icingaweb',
-                'show_stacktraces',
-                $displayExceptions
-            );
-        }
 
         $this->frontController->setParams(
             array(
