@@ -3,28 +3,35 @@
 
 namespace Icinga\File;
 
-use DOMPDF;
-use DOMDocument;
-use DOMXPath;
-use Font_Metrics;
+use Dompdf\Autoloader;
+use Dompdf\Dompdf;
+use Dompdf\Options;
 use Icinga\Application\Icinga;
-use Icinga\Web\StyleSheet;
-use Icinga\Web\Url;
 use Icinga\Exception\ProgrammingError;
+use Icinga\Web\Url;
 
-require_once 'dompdf/dompdf_config.inc.php';
-require_once 'dompdf/include/autoload.inc.php';
+call_user_func(function () {
+    /**
+     * @package dompdf
+     * @link    http://dompdf.github.com/
+     * @author  Benj Carson <benjcarson@digitaljunkies.ca>
+     * @author  Fabien Ménager <fabien.menager@gmail.com>
+     * @author  Alexander A. Klimov <alexander.klimov@icinga.com>
+     * @license http://www.gnu.org/copyleft/lesser.html GNU Lesser General Public License
+     */
 
-class Pdf extends DOMPDF
+    $baseDir = __DIR__ . '/../../vendor/dompdf';
+
+    require_once "$baseDir/lib/html5lib/Parser.php";
+    require_once "$baseDir/lib/php-font-lib/src/FontLib/Autoloader.php";
+    require_once "$baseDir/lib/php-svg-lib/src/autoload.php";
+    require_once "$baseDir/src/Autoloader.php";
+
+    Autoloader::register();
+});
+
+class Pdf
 {
-    public $paginateTable = false;
-
-    public function __construct()
-    {
-        $this->set_paper('A4', 'portrait');
-        parent::__construct();
-    }
-
     protected function assertNoHeadersSent()
     {
         if (headers_sent()) {
@@ -49,13 +56,20 @@ class Pdf extends DOMPDF
         $layout->content = $controller->getResponse();
         $html = $layout->render();
         $imgDir = Url::fromPath('img');
-        $html = preg_replace('~src="' . $imgDir . '/~', 'src="' . Icinga::app()->getBootstrapDirectory() . '/img/', $html);
-        $this->load_html($html);
-        $this->render();
+        $html = preg_replace(
+            '~src="' . $imgDir . '/~',
+            'src="' . Icinga::app()->getBootstrapDirectory() . '/img/',
+            $html
+        );
+        $options = new Options();
+        $options->set('defaultPaperSize', 'A4');
+        $dompdf = new Dompdf($options);
+        $dompdf->loadHtml($html);
+        $dompdf->render();
         $request = $controller->getRequest();
-        $this->stream(
+        $dompdf->stream(
             sprintf(
-                '%s-%s-%d.pdf',
+                '%s-%s-%d',
                 $request->getControllerName(),
                 $request->getActionName(),
                 time()
