@@ -18,6 +18,7 @@ use Icinga\Data\Inspection;
 use Icinga\Data\Selectable;
 use Icinga\Data\Sortable;
 use Icinga\Exception\ProgrammingError;
+use Icinga\Web\Url;
 
 /**
  * Encapsulate LDAP connections and query creation
@@ -152,6 +153,13 @@ class LdapConnection implements Selectable, Inspectable
      */
     protected $discoverySuccess;
 
+    /**
+     * The cause of the discovery's failure
+     *
+     * @var Exception|null
+     */
+    private $discoveryError;
+
     /**
      * Whether the current connection is encrypted
      *
@@ -249,11 +257,13 @@ class LdapConnection implements Selectable, Inspectable
             try {
                 $this->capabilities = LdapCapabilities::discoverCapabilities($this);
                 $this->discoverySuccess = true;
+                $this->discoveryError = null;
             } catch (LdapException $e) {
                 Logger::debug($e);
                 Logger::warning('LADP discovery failed, assuming default LDAP capabilities.');
                 $this->capabilities = new LdapCapabilities(); // create empty default capabilities
                 $this->discoverySuccess = false;
+                $this->discoveryError = $e;
             }
         }
 
@@ -274,6 +284,16 @@ class LdapConnection implements Selectable, Inspectable
         return $this->discoverySuccess;
     }
 
+    /**
+     * Get discovery error if any
+     *
+     * @return Exception|null
+     */
+    public function getDiscoveryError()
+    {
+        return $this->discoveryError;
+    }
+
     /**
      * Return whether the current connection is encrypted
      *
@@ -316,11 +336,11 @@ class LdapConnection implements Selectable, Inspectable
         $success = @ldap_bind($ds, $this->bindDn, $this->bindPw);
         if (! $success) {
             throw new LdapException(
-                'LDAP connection to %s:%s (%s / %s) failed: %s',
-                $this->hostname,
-                $this->port,
+                'LDAP bind (%s / %s) to %s with default port %s failed: %s',
                 $this->bindDn,
                 '***' /* $this->bindPw */,
+                $this->hostname,
+                $this->port,
                 ldap_error($ds)
             );
         }
@@ -747,7 +767,7 @@ class LdapConnection implements Selectable, Inspectable
             $query,
             array_values($fields),
             0,
-            $serverSorting && $limit ? $offset + $limit : 0
+            ($serverSorting || ! $query->hasOrder()) && $limit ? $offset + $limit : 0
         );
         if ($results === false) {
             if (ldap_errno($ds) === self::LDAP_NO_SUCH_OBJECT) {
@@ -803,8 +823,11 @@ class LdapConnection implements Selectable, Inspectable
             && ($entry = ldap_next_entry($ds, $entry))
         );
 
-        if (! $serverSorting && $query->hasOrder()) {
-            uasort($entries, array($query, 'compare'));
+        if (! $serverSorting) {
+            if ($query->hasOrder()) {
+                uasort($entries, array($query, 'compare'));
+            }
+
             if ($limit && $count > $limit) {
                 $entries = array_splice($entries, $query->hasOffset() ? $query->getOffset() : 0, $limit);
             }
@@ -882,7 +905,7 @@ class LdapConnection implements Selectable, Inspectable
                 $query,
                 array_values($fields),
                 0,
-                $serverSorting && $limit ? $offset + $limit : 0
+                ($serverSorting || ! $query->hasOrder()) && $limit ? $offset + $limit : 0
             );
             if ($results === false) {
                 if (ldap_errno($ds) === self::LDAP_NO_SUCH_OBJECT) {
@@ -944,8 +967,7 @@ class LdapConnection implements Selectable, Inspectable
                         );
                     }
                 }
-            } while (
-                (! $serverSorting || $limit === 0 || $limit !== count($entries))
+            } while ((! $serverSorting || $limit === 0 || $limit !== count($entries))
                 && ($entry = ldap_next_entry($ds, $entry))
             );
 
@@ -975,8 +997,11 @@ class LdapConnection implements Selectable, Inspectable
             ldap_search($ds, $query->getBase() ?: $this->getDn(), (string) $query);
         }
 
-        if (! $serverSorting && $query->hasOrder()) {
-            uasort($entries, array($query, 'compare'));
+        if (! $serverSorting) {
+            if ($query->hasOrder()) {
+                uasort($entries, array($query, 'compare'));
+            }
+
             if ($limit && $count > $limit) {
                 $entries = array_splice($entries, $query->hasOffset() ? $query->getOffset() : 0, $limit);
             }
@@ -1053,8 +1078,7 @@ class LdapConnection implements Selectable, Inspectable
             }
         }
 
-        if (
-            $unfoldAttribute !== null
+        if ($unfoldAttribute !== null
             && isset($cleanedAttributes[$unfoldAttribute])
             && is_array($cleanedAttributes[$unfoldAttribute])
         ) {
@@ -1110,7 +1134,6 @@ class LdapConnection implements Selectable, Inspectable
             if ($attributeOctets >= 127) {
                 // Use the indefinite form of the length octets (the long form would be another option)
                 $attributeType = '0440' . $attributeType . '0000';
-
             } else {
                 $attributeType = '04' . str_pad(dechex($attributeOctets), 2, '0', STR_PAD_LEFT) . $attributeType;
             }
@@ -1158,7 +1181,18 @@ class LdapConnection implements Selectable, Inspectable
         $hostname = $this->hostname;
         if ($this->encryption === static::LDAPS) {
             $info->write('Connect using LDAPS');
-            $hostname = 'ldaps://' . $hostname;
+            $ldapUrls = explode(' ', $hostname);
+            if (count($ldapUrls) > 1) {
+                foreach ($ldapUrls as & $uri) {
+                    if (strpos($uri, '://') === false) {
+                        $uri = 'ldaps://' . $uri;
+                    }
+                }
+
+                $hostname = implode(' ', $ldapUrls);
+            } else {
+                $hostname = 'ldaps://' . $hostname;
+            }
         }
 
         $ds = ldap_connect($hostname, $this->port);
@@ -1177,7 +1211,6 @@ class LdapConnection implements Selectable, Inspectable
             if (! ldap_start_tls($ds)) {
                 throw new LdapException('LDAP STARTTLS failed: %s', ldap_error($ds));
             }
-
         } elseif ($this->encryption !== static::LDAPS) {
             $this->encrypted = false;
             $info->write('Connect without encryption');
@@ -1213,12 +1246,27 @@ class LdapConnection implements Selectable, Inspectable
         $scope = $query->getScope();
 
         if (Logger::getInstance()->getLevel() === Logger::DEBUG) {
-            // We're checking the level by ourself to avoid rendering the ldapsearch commandline for nothing
+            // We're checking the level by ourselves to avoid rendering the ldapsearch commandline for nothing
             $starttlsParam = $this->encryption === static::STARTTLS ? ' -ZZ' : '';
-            $ldapUrl = ($this->encryption === static::LDAPS ? 'ldaps://' : 'ldap://')
-                . $this->hostname
-                . ($this->port ? ':' . $this->port : '');
 
+            $ldapUrls = array();
+            $defaultScheme = $this->encryption === static::LDAPS ? 'ldaps://' : 'ldap://';
+            foreach (explode(' ', $this->hostname) as $uri) {
+                $url = Url::fromPath($uri);
+                if (! $url->getScheme()) {
+                    $uri = $defaultScheme . $uri . ($this->port ? ':' . $this->port : '');
+                } else {
+                    if ($url->getPort() === null) {
+                        $url->setPort($this->port);
+                    }
+
+                    $uri = $url->getAbsoluteUrl();
+                }
+
+                $ldapUrls[] = $uri;
+            }
+
+            $bindParams = '';
             if ($this->bound) {
                 $bindParams = ' -D "' . $this->bindDn . '"' . ($this->bindPw ? ' -W' : '');
             }
@@ -1236,7 +1284,7 @@ class LdapConnection implements Selectable, Inspectable
             Logger::debug("Issueing LDAP search. Use '%s' to reproduce.", sprintf(
                 'ldapsearch -P 3%s -H "%s"%s -b "%s" -s "%s" -z %u -l %u -a "%s"%s%s%s',
                 $starttlsParam,
-                $ldapUrl,
+                implode(' ', $ldapUrls),
                 $bindParams,
                 $baseDn,
                 $scope,
@@ -1249,7 +1297,7 @@ class LdapConnection implements Selectable, Inspectable
             ));
         }
 
-        switch($scope) {
+        switch ($scope) {
             case LdapQuery::SCOPE_SUB:
                 $function = 'ldap_search';
                 break;
@@ -1456,11 +1504,11 @@ class LdapConnection implements Selectable, Inspectable
         // Try a bind-command with the given user credentials, this must not fail
         $success = @ldap_bind($ds, $this->bindDn, $this->bindPw);
         $msg = sprintf(
-            'LDAP bind to %s:%s (%s / %s)',
-            $this->hostname,
-            $this->port,
+            'LDAP bind (%s / %s) to %s with default port %s',
             $this->bindDn,
-            '***' /* $this->bindPw */
+            '***' /* $this->bindPw */,
+            $this->hostname,
+            $this->port
         );
         if (! $success) {
             // ldap_error does not return any proper error messages in case of certificate errors. Connecting
