@@ -24,6 +24,7 @@ class JavaScript
         'js/icinga/timezone.js',
         'js/icinga/behavior/application-state.js',
         'js/icinga/behavior/autofocus.js',
+        'js/icinga/behavior/detach.js',
         'js/icinga/behavior/tooltip.js',
         'js/icinga/behavior/sparkline.js',
         'js/icinga/behavior/tristate.js',
@@ -31,6 +32,8 @@ class JavaScript
         'js/icinga/behavior/navigation.js',
         'js/icinga/behavior/form.js',
         'js/icinga/behavior/actiontable.js',
+        'js/icinga/behavior/flyover.js',
+        'js/icinga/behavior/expandable.js',
         'js/icinga/behavior/selectable.js'
     );
 
@@ -98,13 +101,13 @@ class JavaScript
         $request = Icinga::app()->getRequest();
         $noCache = $request->getHeader('Cache-Control') === 'no-cache' || $request->getHeader('Pragma') === 'no-cache';
 
+        header('Cache-Control: public');
         if (! $noCache && FileCache::etagMatchesFiles($files)) {
             header("HTTP/1.1 304 Not Modified");
             return;
         } else {
             $etag = FileCache::etagForFiles($files);
         }
-        header('Cache-Control: public');
         header('ETag: "' . $etag . '"');
         header('Content-Type: application/javascript');
 
