@@ -36,7 +36,9 @@ EOT;
      */
     private $dropdownTpl = <<< 'EOT'
 <li class="dropdown-nav-item">
-  <a href="#" class="dropdown-toggle" title="{TITLE}" aria-label="{TITLE}"><i aria-hidden="true" class="icon-down-open"></i></a>
+  <a href="#" class="dropdown-toggle" title="{TITLE}" aria-label="{TITLE}">
+    <i aria-hidden="true" class="icon-down-open"></i>
+  </a>
   <ul class="nav">
     {TABS}
   </ul>
@@ -49,7 +51,7 @@ EOT;
      * @var string
      */
     private $closeTpl = <<< 'EOT'
-<li class="dropdown" style="float: right;">
+<li style="float: right;">
   <a href="#" title="{TITLE}" aria-label="{TITLE}" class="close-container-control">
     <i aria-hidden="true" class="icon-cancel"></i>
   </a>
@@ -315,7 +317,7 @@ EOT;
 
     private function renderRefreshTab()
     {
-        $url = Icinga::app()->getRequest()->getUrl();
+        $url = Url::fromRequest();
         $tab = $this->get($this->getActiveName());
 
         if ($tab !== null) {
@@ -330,7 +332,7 @@ EOT;
             $caption = t('Content');
         }
 
-        $label = t(sprintf('Refresh the %s', $caption));
+        $label = sprintf(t('Refresh the %s'), $caption);
         $title = $label;
 
         $tpl = str_replace(
