@@ -79,10 +79,8 @@ class ListController extends Controller
             'host_passive_checks_enabled'
         ), $this->addColumns()));
         $this->applyRestriction('monitoring/filter/objects', $hosts);
-        $this->filterQuery($hosts);
 
         $this->setupPaginationControl($hosts);
-        $this->setupLimitControl();
         $this->setupSortControl(array(
             'host_severity'             => $this->translate('Severity'),
             'host_state'                => $this->translate('Current State'),
@@ -91,6 +89,8 @@ class ListController extends Controller
             'host_last_check'           => $this->translate('Last Check'),
             'host_last_state_change'    => $this->translate('Last State Change')
         ), $hosts);
+        $this->filterQuery($hosts);
+        $this->setupLimitControl();
 
         $stats = $this->backend->select()->from('hoststatussummary', array(
             'hosts_total',
@@ -106,7 +106,6 @@ class ListController extends Controller
         $this->applyRestriction('monitoring/filter/objects', $stats);
 
         $summary = $hosts->getQuery()->queryServiceProblemSummary();
-        $this->applyRestriction('monitoring/filter/objects', $summary);
 
         $this->view->hosts = $hosts;
         $this->view->stats = $stats;
@@ -160,10 +159,8 @@ class ListController extends Controller
             'service_passive_checks_enabled'
         ), $this->addColumns()));
         $this->applyRestriction('monitoring/filter/objects', $services);
-        $this->filterQuery($services);
 
         $this->setupPaginationControl($services);
-        $this->setupLimitControl();
         $this->setupSortControl(array(
             'service_severity'          => $this->translate('Service Severity'),
             'service_state'             => $this->translate('Current Service State'),
@@ -176,6 +173,8 @@ class ListController extends Controller
             'host_address'              => $this->translate('Host Address'),
             'host_last_check'           => $this->translate('Last Host Check')
         ), $services);
+        $this->filterQuery($services);
+        $this->setupLimitControl();
 
         $stats = $this->backend->select()->from('servicestatussummary', array(
             'services_critical',
@@ -229,6 +228,7 @@ class ListController extends Controller
             'is_fixed'        => 'downtime_is_fixed',
             'is_in_effect'    => 'downtime_is_in_effect',
             'entry_time'      => 'downtime_entry_time',
+            'name'            => 'downtime_name',
             'host_state',
             'service_state',
             'host_name',
@@ -237,10 +237,8 @@ class ListController extends Controller
             'service_display_name'
         ));
         $this->applyRestriction('monitoring/filter/objects', $downtimes);
-        $this->filterQuery($downtimes);
 
         $this->setupPaginationControl($downtimes);
-        $this->setupLimitControl();
         $this->setupSortControl(array(
             'downtime_is_in_effect'     => $this->translate('Is In Effect'),
             'host_display_name'         => $this->translate('Host'),
@@ -253,6 +251,8 @@ class ListController extends Controller
             'downtime_scheduled_end'    => $this->translate('Scheduled End'),
             'downtime_duration'         => $this->translate('Duration')
         ), $downtimes);
+        $this->filterQuery($downtimes);
+        $this->setupLimitControl();
 
         $this->view->downtimes = $downtimes;
 
@@ -276,23 +276,23 @@ class ListController extends Controller
         $this->setAutorefreshInterval(15);
 
         $notifications = $this->backend->select()->from('notification', array(
-            'host_name',
-            'service_description',
-            'notification_output',
-            'notification_contact_name',
-            'notification_start_time',
-            'notification_state',
             'host_display_name',
+            'host_name',
+            'notification_contact_name',
+            'notification_output',
+            'notification_state',
+            'notification_timestamp',
+            'service_description',
             'service_display_name'
         ));
         $this->applyRestriction('monitoring/filter/objects', $notifications);
-        $this->filterQuery($notifications);
 
         $this->setupPaginationControl($notifications);
-        $this->setupLimitControl();
         $this->setupSortControl(array(
-            'notification_start_time' => $this->translate('Notification Start')
+            'notification_timestamp' => $this->translate('Notification Start')
         ), $notifications);
+        $this->filterQuery($notifications);
+        $this->setupLimitControl();
 
         $this->view->notifications = $notifications;
     }
@@ -317,16 +317,16 @@ class ListController extends Controller
             'contact_notify_host_timeperiod'
         ));
         $this->applyRestriction('monitoring/filter/objects', $contacts);
-        $this->filterQuery($contacts);
 
         $this->setupPaginationControl($contacts);
-        $this->setupLimitControl();
         $this->setupSortControl(array(
             'contact_name'  => $this->translate('Name'),
             'contact_alias' => $this->translate('Alias'),
             'contact_email' => $this->translate('Email'),
             'contact_pager' => $this->translate('Pager Address / Number')
         ), $contacts);
+        $this->filterQuery($contacts);
+        $this->setupLimitControl();
 
         $this->view->contacts = $contacts;
     }
@@ -343,7 +343,9 @@ class ListController extends Controller
         $form->render();
         $this->view->form = $form;
 
-        $this->params->remove('view');
+        $this->params
+            ->remove('view')
+            ->remove('format');
         $orientation = $this->params->shift('vertical', 0) ? 'vertical' : 'horizontal';
 /*
         $orientationBox = new SelectBox(
@@ -388,14 +390,14 @@ class ListController extends Controller
             'contact_count'
         ));
         $this->applyRestriction('monitoring/filter/objects', $contactGroups);
-        $this->filterQuery($contactGroups);
 
         $this->setupPaginationControl($contactGroups);
-        $this->setupLimitControl();
         $this->setupSortControl(array(
             'contactgroup_name'     => $this->translate('Contactgroup Name'),
             'contactgroup_alias'    => $this->translate('Contactgroup Alias')
         ), $contactGroups);
+        $this->filterQuery($contactGroups);
+        $this->setupLimitControl();
 
         $this->view->contactGroups = $contactGroups;
     }
@@ -422,16 +424,15 @@ class ListController extends Controller
             'type'       => 'comment_type',
             'persistent' => 'comment_is_persistent',
             'expiration' => 'comment_expiration',
+            'name'       => 'comment_name',
             'host_name',
             'service_description',
             'host_display_name',
             'service_display_name'
         ));
         $this->applyRestriction('monitoring/filter/objects', $comments);
-        $this->filterQuery($comments);
 
         $this->setupPaginationControl($comments);
-        $this->setupLimitControl();
         $this->setupSortControl(
             array(
                 'comment_timestamp'     => $this->translate('Comment Timestamp'),
@@ -442,6 +443,8 @@ class ListController extends Controller
             ),
             $comments
         );
+        $this->filterQuery($comments);
+        $this->setupLimitControl();
 
         $this->view->comments = $comments;
 
@@ -478,15 +481,14 @@ class ListController extends Controller
             'services_warning_unhandled'
         ));
         $this->applyRestriction('monitoring/filter/objects', $serviceGroups);
-        $this->filterQuery($serviceGroups);
 
         $this->setupPaginationControl($serviceGroups);
-        $this->setupLimitControl();
         $this->setupSortControl(array(
-            'services_severity'     => $this->translate('Severity'),
             'servicegroup_alias'    => $this->translate('Service Group Name'),
             'services_total'        => $this->translate('Total Services')
         ), $serviceGroups);
+        $this->filterQuery($serviceGroups);
+        $this->setupLimitControl();
 
         $this->view->serviceGroups = $serviceGroups;
     }
@@ -525,16 +527,15 @@ class ListController extends Controller
             'services_warning_unhandled'
         ));
         $this->applyRestriction('monitoring/filter/objects', $hostGroups);
-        $this->filterQuery($hostGroups);
 
         $this->setupPaginationControl($hostGroups);
-        $this->setupLimitControl();
         $this->setupSortControl(array(
-            'hosts_severity'    => $this->translate('Severity'),
             'hostgroup_alias'   => $this->translate('Host Group Name'),
             'hosts_total'       => $this->translate('Total Hosts'),
             'services_total'    => $this->translate('Total Services')
         ), $hostGroups);
+        $this->filterQuery($hostGroups);
+        $this->setupLimitControl();
 
         $this->view->hostGroups = $hostGroups;
     }
@@ -548,6 +549,7 @@ class ListController extends Controller
         );
 
         $query = $this->backend->select()->from('eventhistory', array(
+            'id',
             'host_name',
             'host_display_name',
             'service_description',
@@ -560,13 +562,13 @@ class ListController extends Controller
         ));
 
         $this->applyRestriction('monitoring/filter/objects', $query);
-        $this->filterQuery($query);
         $this->view->history = $query;
 
-        $this->setupLimitControl();
         $this->setupSortControl(array(
             'timestamp' => $this->translate('Occurence')
         ), $query);
+        $this->filterQuery($query);
+        $this->setupLimitControl();
     }
 
     public function servicegridAction()
