@@ -655,11 +655,15 @@ abstract class IdoQuery extends DbQuery
                     '(CASE WHEN $1 ~ \'(?:[0-9]{1,3}\\\\.){3}[0-9]{1,3}\' THEN $1::inet - \'0.0.0.0\' ELSE NULL END)',
                     $column
                 );
-                $column = preg_replace(
-                    '/UNIX_TIMESTAMP(\((?>[^()]|(?-1))*\))/i',
-                    'CASE WHEN ($1 < \'1970-01-03 00:00:00+00\'::timestamp with time zone) THEN 0 ELSE UNIX_TIMESTAMP($1) END',
-                    $column
-                );
+                if (version_compare($this->getIdoVersion(), '1.14.2', '>=')) {
+                    $column = str_replace('NOW()', 'NOW() AT TIME ZONE \'UTC\'', $column);
+                } else {
+                    $column = preg_replace(
+                        '/UNIX_TIMESTAMP(\((?>[^()]|(?-1))*\))/i',
+                        'CASE WHEN ($1 < \'1970-01-03 00:00:00+00\'::timestamp with time zone) THEN 0 ELSE UNIX_TIMESTAMP($1) END',
+                        $column
+                    );
+                }
             }
         }
     }
@@ -676,7 +680,9 @@ abstract class IdoQuery extends DbQuery
 
         foreach (Hook::all('monitoring/idoQueryExtension') as $hook) {
             $extensions = $hook->extendColumnMap($this);
-            if (! is_array($extensions)) continue;
+            if (! is_array($extensions)) {
+                continue;
+            }
 
             foreach ($extensions as $vTable => $cols) {
                 if (! array_key_exists($vTable, $this->columnMap)) {
@@ -958,6 +964,12 @@ abstract class IdoQuery extends DbQuery
             $leftcol = 'h.host_object_id';
         }
 
+        $mapped = $this->getMappedField($leftcol);
+        if ($mapped !== null) {
+            $this->requireColumn($leftcol);
+            $leftcol = $mapped;
+        }
+
         $joinOn = sprintf(
             $this->customVarsJoinTemplate,
             $leftcol,
@@ -1146,6 +1158,9 @@ abstract class IdoQuery extends DbQuery
 
         $groupedTables = array();
         foreach ($this->groupBase as $baseTable => $aliasedPks) {
+            if (! $this->hasJoinedVirtualTable($baseTable)) {
+                continue;
+            }
             $groupedTables[$baseTable] = true;
             foreach ($aliasedPks as $aliasedPk) {
                 $group[] = $aliasedPk;
@@ -1154,6 +1169,14 @@ abstract class IdoQuery extends DbQuery
 
         foreach (new ColumnFilterIterator($this->columns) as $desiredAlias => $desiredColumn) {
             $alias = is_string($desiredAlias) ? $this->customAliasToAlias($desiredAlias) : $desiredColumn;
+            if ($this->isCustomVar($alias) && $this->getDatasource()->getDbType() === 'pgsql') {
+                $table = $this->customVars[$alias];
+                if (! isset($groupedTables[$table])) {
+                    $group[] = $this->getCustomvarColumnName($alias);
+                    $groupedTables[$table] = true;
+                }
+                continue;
+            }
             $table = $this->aliasToTableName($alias);
             if ($table && !isset($groupedTables[$table]) && (
                 in_array($table, $joinedOrigins, true) || $this->getDatasource()->getDbType() === 'pgsql')
@@ -1164,6 +1187,14 @@ abstract class IdoQuery extends DbQuery
 
         if (! empty($group) && $this->getDatasource()->getDbType() === 'pgsql') {
             foreach (new ColumnFilterIterator($this->orderColumns) as $alias) {
+                if ($this->isCustomVar($alias)) {
+                    $table = $this->customVars[$alias];
+                    if (! isset($groupedTables[$table])) {
+                        $group[] = $this->getCustomvarColumnName($alias);
+                        $groupedTables[$table] = true;
+                    }
+                    continue;
+                }
                 $table = $this->aliasToTableName($alias);
                 if ($table && !isset($groupedTables[$table])
                     && !in_array($this->getMappedField($alias), $this->columns, true)
@@ -1215,8 +1246,7 @@ abstract class IdoQuery extends DbQuery
     {
         // TODO: For god's sake, make this being a mapping
         //       (instead of matching a ton of properties using a ridiculous long switch case)
-        switch ($table)
-        {
+        switch ($table) {
             case 'instances':
                 return $this->instance_id;
             case 'objects':
