@@ -173,19 +173,12 @@ abstract class MonitoredObject implements Filterable
      */
     abstract protected function getDataView();
 
-    /**
-     * Get the notes for this monitored object
-     *
-     * @return string The notes as a string
-     */
-    public abstract function getNotes();
-
     /**
      * Get all note urls configured for this monitored object
      *
      * @return array All note urls as a string
      */
-    public abstract function getNotesUrls();
+    abstract public function getNotesUrls();
 
     /**
      * {@inheritdoc}
@@ -322,16 +315,12 @@ abstract class MonitoredObject implements Filterable
      */
     public function fetchComments()
     {
-        if ($this->backend->is('livestatus')) {
-            $this->comments = array();
-            return $this;
-        }
-
         $commentsView = $this->backend->select()->from('comment', array(
             'author'            => 'comment_author_name',
             'comment'           => 'comment_data',
             'expiration'        => 'comment_expiration',
             'id'                => 'comment_internal_id',
+            'name'              => 'comment_name',
             'persistent'        => 'comment_is_persistent',
             'timestamp'         => 'comment_timestamp',
             'type'              => 'comment_type'
@@ -383,11 +372,6 @@ abstract class MonitoredObject implements Filterable
      */
     public function fetchContactgroups()
     {
-        if ($this->backend->is('livestatus')) {
-            $this->contactgroups = array();
-            return $this;
-        }
-
         $contactsGroups = $this->backend->select()->from('contactgroup', array(
             'contactgroup_name',
             'contactgroup_alias'
@@ -410,11 +394,6 @@ abstract class MonitoredObject implements Filterable
      */
     public function fetchContacts()
     {
-        if ($this->backend->is('livestatus')) {
-            $this->contacts = array();
-            return $this;
-        }
-
         $contacts = $this->backend->select()->from('contact', array(
             'contact_name',
             'contact_alias',
@@ -439,13 +418,8 @@ abstract class MonitoredObject implements Filterable
      */
     public function fetchCustomvars()
     {
-        if ($this->backend->is('livestatus')) {
-            $this->customvars = array();
-            return $this;
-        }
-
         $blacklist = array();
-        $blacklistPattern = '/^(.*pw.*|.*pass.*|community)$/i';
+        $blacklistPattern = '';
 
         if (($blacklistConfig = Config::module('monitoring')->get('security', 'protected_customvars', '')) !== '') {
             foreach (explode(',', $blacklistConfig) as $customvar) {
@@ -468,7 +442,10 @@ abstract class MonitoredObject implements Filterable
 
         $this->customvars = $customvars;
         $this->hideBlacklistedProperties();
-        $this->customvars = $this->obfuscateCustomVars($this->customvars, $blacklistPattern);
+
+        if ($blacklistPattern) {
+            $this->customvars = $this->obfuscateCustomVars($this->customvars, $blacklistPattern);
+        }
 
         return $this;
     }
@@ -584,19 +561,20 @@ abstract class MonitoredObject implements Filterable
     public function fetchDowntimes()
     {
         $downtimes = $this->backend->select()->from('downtime', array(
-            'id'                => 'downtime_internal_id',
-            'objecttype'        => 'object_type',
-            'comment'           => 'downtime_comment',
             'author_name'       => 'downtime_author_name',
-            'start'             => 'downtime_start',
-            'scheduled_start'   => 'downtime_scheduled_start',
-            'scheduled_end'     => 'downtime_scheduled_end',
-            'end'               => 'downtime_end',
+            'comment'           => 'downtime_comment',
             'duration'          => 'downtime_duration',
-            'is_flexible'       => 'downtime_is_flexible',
+            'end'               => 'downtime_end',
+            'entry_time'        => 'downtime_entry_time',
+            'id'                => 'downtime_internal_id',
             'is_fixed'          => 'downtime_is_fixed',
+            'is_flexible'       => 'downtime_is_flexible',
             'is_in_effect'      => 'downtime_is_in_effect',
-            'entry_time'        => 'downtime_entry_time'
+            'name'              => 'downtime_name',
+            'objecttype'        => 'object_type',
+            'scheduled_end'     => 'downtime_scheduled_end',
+            'scheduled_start'   => 'downtime_scheduled_start',
+            'start'             => 'downtime_start'
         ))
             ->where('object_type', $this->type)
             ->order('downtime_is_in_effect', 'DESC')
@@ -625,6 +603,7 @@ abstract class MonitoredObject implements Filterable
             ->from(
                 'eventhistory',
                 array(
+                    'id',
                     'object_type',
                     'host_name',
                     'host_display_name',
@@ -752,7 +731,7 @@ abstract class MonitoredObject implements Filterable
      * Find all occurences of http links, separated by whitespaces and quoted
      * by single or double-ticks.
      *
-     * @link http://docs.icinga.org/latest/de/objectdefinitions.html
+     * @link http://docs.icinga.com/latest/de/objectdefinitions.html
      *
      * @param   string  $urlString  A string containing one or more urls
      * @return  array                   Array of urls as strings
@@ -873,13 +852,17 @@ abstract class MonitoredObject implements Filterable
                     $this->fetchContacts();
                 }
 
-                return array_map(function ($el) { return $el->contact_name; }, $this->contacts);
+                return array_map(function ($el) {
+                    return $el->contact_name;
+                }, $this->contacts);
             } elseif ($name === 'contactgroup_name') {
                 if ($this->contactgroups === null) {
                     $this->fetchContactgroups();
                 }
 
-                return array_map(function ($el) { return $el->contactgroup_name; }, $this->contactgroups);
+                return array_map(function ($el) {
+                    return $el->contactgroup_name;
+                }, $this->contactgroups);
             } elseif ($name === 'hostgroup_name') {
                 if ($this->hostgroups === null) {
                     $this->fetchHostgroups();
