@@ -1,4 +1,4 @@
-/*! Icinga Web 2 | (c) 2013-2015 Icinga Development Team | GPLv2+ */
+/*! Icinga Web 2 | (c) 2014 Icinga Development Team | GPLv2+ */
 
 (function(Icinga, $) {
 
@@ -15,13 +15,17 @@
 
     Sparkline.prototype.onRendered = function(e) {
         $(e.target).find('.sparkline').each(function() {
-            $(this).sparkline('html', {
-                enableTagOptions: true,
-                disableTooltips: true
-            });
+            setTimeout(sparkline, 0, $(this));
         });
     };
 
     Icinga.Behaviors.Sparkline = Sparkline;
 
+    function sparkline(selector) {
+        selector.sparkline('html', {
+            enableTagOptions: true,
+            disableTooltips: true
+        });
+    }
+
 })(Icinga, jQuery);
