@@ -1,4 +1,4 @@
-/*! Icinga Web 2 | (c) 2013-2015 Icinga Development Team | GPLv2+ */
+/*! Icinga Web 2 | (c) 2014 Icinga Development Team | GPLv2+ */
 
 /**
  * Icinga.Events
@@ -33,10 +33,16 @@
         // TODO: What's this?
         applyHandlers: function (event) {
             var $target = $(event.target);
-            var self = event.data.self;
-            var icinga = self.icinga;
+            var _this = event.data.self;
+            var icinga = _this.icinga;
 
-            if (self.initializeModules) {
+            if (! icinga) {
+                // Attempt to catch a rare error, race condition, whatever
+                console.log('Got no icinga in applyHandlers');
+                return;
+            }
+
+            if (_this.initializeModules) {
                 var loaded = false;
                 var moduleName = $target.data('icingaModule');
                 if (moduleName) {
@@ -57,7 +63,7 @@
                     // so we need to ensure that it is called the first time they are
                     // initialized
                     event.stopImmediatePropagation();
-                    self.initializeModules = false;
+                    _this.initializeModules = false;
 
                     var $container = $target.closest('.container');
                     if (! $container.length) {
@@ -73,7 +79,7 @@
                     return false;
                 }
             } else {
-                self.initializeModules = true;
+                _this.initializeModules = true;
             }
 
             $('.dashboard > div', $target).each(function(idx, el) {
@@ -87,13 +93,7 @@
             var $searchField = $('#menu input.search', $target);
             // Remember initial search field value if any
             if ($searchField.length && $searchField.val().length) {
-                self.searchValue = $searchField.val();
-            }
-
-            if (icinga.ui.isOneColLayout()) {
-                icinga.ui.disableCloseButtons();
-            } else {
-                icinga.ui.enableCloseButtons();
+                _this.searchValue = $searchField.val();
             }
         },
 
@@ -105,7 +105,9 @@
             // Note: It is important that this is the first handler for this event!
             $(document).on('rendered', { self: this }, this.applyHandlers);
 
-            $.each(self.icinga.behaviors, function (name, behavior) {
+            $(document).on('visibilitychange', { self: this }, this.onVisibilityChange);
+
+            $.each(this.icinga.behaviors, function (name, behavior) {
                 behavior.bind($(document));
             });
 
@@ -129,6 +131,7 @@
             $(document).on('click', 'a', { self: this }, this.linkClicked);
             $(document).on('click', 'tr[href]', { self: this }, this.linkClicked);
 
+            $(document).on('click', 'input[type="submit"], button[type="submit"]', this.rememberSubmitButton);
             // We catch all form submit events
             $(document).on('submit', 'form', { self: this }, this.submitForm);
 
@@ -144,6 +147,8 @@
 
             $(document).on('click', '.tree .handle', { self: this }, this.treeNodeToggle);
 
+            $(document).on('click', '#search + .search-reset', this.clearSearch);
+
             // TBD: a global autocompletion handler
             // $(document).on('keyup', 'form.auto input', this.formChangeDelayed);
             // $(document).on('change', 'form.auto input', this.formChanged);
@@ -171,6 +176,18 @@
             icinga.destroy();
         },
 
+        onVisibilityChange: function (event) {
+            var icinga = event.data.self.icinga;
+
+            if (document.visibilityState === undefined || document.visibilityState === 'visible') {
+                icinga.loader.autorefreshSuspended = false;
+                icinga.logger.debug('Page visible, enabling auto-refresh');
+            } else {
+                icinga.loader.autorefreshSuspended = true;
+                icinga.logger.debug('Page invisible, disabling auto-refresh');
+            }
+        },
+
         /**
          * A scroll event happened in one of our containers
          */
@@ -189,12 +206,18 @@
         },
 
         autoSubmitSearch: function(event) {
-            var self = event.data.self;
-            if ($('#menu input.search').val() === self.searchValue) {
+            var _this = event.data.self;
+            if ($('#menu input.search').val() === _this.searchValue) {
                 return;
             }
-            self.searchValue = $('#menu input.search').val();
-            return self.autoSubmitForm(event);
+            _this.searchValue = $('#menu input.search').val();
+            return _this.autoSubmitForm(event);
+        },
+
+        rememberSubmitButton: function(e) {
+            var $button = $(this);
+            var $form = $button.closest('form');
+            $form.data('submitButton', $button);
         },
 
         autoSubmitForm: function (event) {
@@ -205,8 +228,8 @@
          *
          */
         submitForm: function (event, autosubmit) {
-            var self   = event.data.self;
-            var icinga = self.icinga;
+            var _this   = event.data.self;
+            var icinga = _this.icinga;
             // .closest is not required unless subelements to trigger this
             var $form = $(event.currentTarget).closest('form');
             var url = $form.attr('action');
@@ -217,6 +240,14 @@
             var $target;
             var data;
 
+            var $rememberedSubmittButton = $form.data('submitButton');
+            if (typeof $rememberedSubmittButton != 'undefined') {
+                if ($form.has($rememberedSubmittButton)) {
+                    $button = $rememberedSubmittButton;
+                }
+                $form.removeData('submitButton');
+            }
+
             if ($button.length === 0) {
                 var $el;
 
@@ -262,9 +293,9 @@
                     $button.addClass('active');
                 }
 
-                $target = self.getLinkTargetFor($button);
+                $target = _this.getLinkTargetFor($button);
             } else {
-                $target = self.getLinkTargetFor($form);
+                $target = _this.getLinkTargetFor($form);
             }
 
             if (! url) {
@@ -386,7 +417,9 @@
                 }
             }
 
-            icinga.loader.loadUrl(url, $target, data, method).progressTimer = progressTimer;
+            var req = icinga.loader.loadUrl(url, $target, data, method);
+            req.forceFocus = autosubmit ? $(event.currentTarget) : $button.length ? $button : null;
+            req.progressTimer = progressTimer;
 
             event.stopPropagation();
             event.preventDefault();
@@ -410,16 +443,16 @@
          * Someone clicked a link or tr[href]
          */
         linkClicked: function (event) {
-            var self   = event.data.self;
-            var icinga = self.icinga;
+            var _this   = event.data.self;
+            var icinga = _this.icinga;
             var $a = $(this);
             var $eventTarget = $(event.target);
             var href = $a.attr('href');
             var linkTarget = $a.attr('target');
             var $target;
             var formerUrl;
-            var remote = /^(?:[a-z]+:)\/\//;
-            if (href.match(/^(mailto|javascript):/)) {
+            if (href.match(/^(?:(?:mailto|javascript|data):|[a-z]+:\/\/)/)) {
+                event.stopPropagation();
                 return true;
             }
 
@@ -437,11 +470,6 @@
                 }
             }
 
-            // Let remote links pass through
-            if  (href.match(remote)) {
-                return true;
-            }
-
             // window.open is used as return true; didn't work reliable
             if (linkTarget === '_blank' || linkTarget === '_self') {
                 window.open(href, linkTarget);
@@ -472,7 +500,8 @@
 
             // This is an anchor only
             if (href.substr(0, 1) === '#' && href.length > 1
-                && href.substr(1, 1) !== '!') {
+                && href.substr(1, 1) !== '!'
+            ) {
                 icinga.ui.focusElement(href.substr(1), $a.closest('.container'));
                 return;
             }
@@ -485,7 +514,7 @@
             // If link has hash tag...
             if (href.match(/#/)) {
                 if (href === '#') {
-                    if ($a.hasClass('close-toggle')) {
+                    if ($a.hasClass('close-container-control')) {
                         if (! icinga.ui.isOneColLayout()) {
                             var $cont = $a.closest('.container').first();
                             if ($cont.attr('id') === 'col1') {
@@ -499,7 +528,7 @@
                     }
                     return false;
                 }
-                $target = self.getLinkTargetFor($a);
+                $target = _this.getLinkTargetFor($a);
 
                 formerUrl = $target.data('icingaUrl');
                 if (typeof formerUrl !== 'undefined' && formerUrl.split(/#/)[0] === href.split(/#/)[0]) {
@@ -511,7 +540,7 @@
                     return false;
                 }
             } else {
-                $target = self.getLinkTargetFor($a);
+                $target = _this.getLinkTargetFor($a);
             }
 
             // Load link URL
@@ -564,11 +593,11 @@
                 } else if (targetId === '_main') {
                     targetId = 'col1';
                     $target = $('#' + targetId);
-                    self.icinga.ui.layout1col();
+                    this.icinga.ui.layout1col();
                 } else {
                     $target = $('#' + targetId);
                     if (! $target.length) {
-                        self.icinga.logger.warn('Link target "#' + targetId + '" does not exist in DOM.');
+                        this.icinga.logger.warn('Link target "#' + targetId + '" does not exist in DOM.');
                     }
                 }
 
@@ -582,8 +611,12 @@
             return $target;
         },
 
+        clearSearch: function (event) {
+            $(event.target).parent().find('#search').attr('value', '');
+        },
+
         unbindGlobalHandlers: function () {
-            $.each(self.icinga.behaviors, function (name, behavior) {
+            $.each(this.icinga.behaviors, function (name, behavior) {
                 behavior.unbind($(document));
             });
             $(window).off('resize', this.onWindowResize);
@@ -597,6 +630,7 @@
             $(document).off('change', 'form input.autosubmit', this.submitForm);
             $(document).off('focus', 'form select[data-related-radiobtn]', this.autoCheckRadioButton);
             $(document).off('focus', 'form input[data-related-radiobtn]', this.autoCheckRadioButton);
+            $(document).off('visibilitychange', this.onVisibilityChange);
         },
 
         destroy: function() {
