@@ -13,7 +13,7 @@ use Icinga\Test\BaseTestCase;
  *
  * Empty lines in module.info must be ignored if they're not part of the module's description
  *
- * @see https://dev.icinga.org/issues/11831
+ * @see https://dev.icinga.com/issues/11831
  */
 class Bug11831Test extends BaseTestCase
 {
@@ -30,7 +30,8 @@ EOT;
         $reflection = new ReflectionClass($module);
         $prop = $reflection->getProperty('metadataFile');
         $prop->setAccessible(true);
-        $prop->setValue($module, stream_get_meta_data($moduleInfoFile)['uri']);
+        $meta = stream_get_meta_data($moduleInfoFile);
+        $prop->setValue($module, $meta['uri']);
         $this->assertEquals($module->getVersion(), '1.0.0');
         fclose($moduleInfoFile);
     }
